import { readdirSync } from 'fs';
import { resolve, join } from 'path';
import { curryMatch } from './curryMatch';
export function fixture(ava, casesPath, baselinesPath, resultsPath) {
    if (baselinesPath && !resultsPath) {
        throw new Error('baselines and results must be specified together');
    }
    function curry(testfn) {
        return ((title, caseName, run) => {
            const hasTitle = !!run;
            if (!run) {
                // title is optional
                run = caseName;
                caseName = title;
            }
            let d = {
                casePath: resolve(casesPath, caseName)
            };
            if (baselinesPath) {
                d.baselinePath = resolve(baselinesPath, caseName);
                d.resultPath = resolve(resultsPath, caseName);
            }
            return testfn(`${hasTitle ? title + ' ' : ''}${caseName}`, (t) => {
                if (baselinesPath) {
                    d.match = curryMatch(d.baselinePath, d.resultPath, t);
                }
                // Inside each test, user may change the `cwd` of each case
                // so the target code can run correctly.
                // At the end of each test, restore the `cwd` to the original value.
                // doing `try-finally` and `thenable` to handle both sync and async.
                const cwd = process.cwd();
                try {
                    const result = run(t, d);
                    if (result && result.then) {
                        return result.then(r => {
                            process.chdir(cwd);
                            return r;
                        }, e => {
                            process.chdir(cwd);
                            throw e;
                        });
                    }
                }
                finally {
                    process.chdir(cwd);
                }
            });
        });
    }
    function eachCurry(testfn) {
        return (filter, run) => {
            const absCasesPath = resolve(casesPath);
            let dirs = readdirSync(absCasesPath).toString().split(',');
            if (!run) {
                // filter is optional
                run = filter;
            }
            else {
                const regex = filter instanceof RegExp ? filter : new RegExp(filter);
                dirs = dirs.filter(d => regex.test(d));
            }
            const testResultPromises = dirs.map(caseName => {
                const d = {
                    caseName,
                    casePath: join(absCasesPath, caseName)
                };
                if (baselinesPath) {
                    d.baselinePath = resolve(baselinesPath, caseName);
                    d.resultPath = resolve(resultsPath, caseName);
                }
                return testfn(caseName, (t) => {
                    if (baselinesPath) {
                        d.match = curryMatch(d.baselinePath, d.resultPath, t);
                    }
                    // Inside each test, user may change the `cwd` of each case
                    // so the target code can run correctly.
                    // At the end of each test, restore the `cwd` to the original value.
                    // doing `try-finally` and `thenable` to handle both sync and async.
                    const cwd = process.cwd();
                    try {
                        const result = run(t, d);
                        if (result && result.then) {
                            return result.then(r => {
                                process.chdir(cwd);
                                return r;
                            }, e => {
                                process.chdir(cwd);
                                throw e;
                            });
                        }
                    }
                    finally {
                        process.chdir(cwd);
                    }
                });
            });
            // fail if any testResultPromises fail?
            return Promise.all(testResultPromises);
        };
    }
    let fn = curry(ava);
    let others = {
        failing: curry(ava.failing),
        only: curry(ava.only),
        skip: curry(ava.skip),
        todo: ava.todo,
        each: eachCurry(ava)
    };
    for (let key in others) {
        fn[key] = others[key];
    }
    const eachFailing = eachCurry(ava.failing);
    fn.failing.only = others.only;
    fn.failing.only.skip = others.skip;
    fn.failing.skip = others.skip;
    fn.failing.skip.only = others.only;
    fn.only.failing = others.failing;
    fn.only.failing.skip = others.skip;
    fn.only.skip = others.skip;
    fn.only.skip.failing = others.failing;
    fn.only.each = others.each;
    fn.only.each.failing = eachFailing;
    fn.skip.failing = others.failing;
    fn.skip.failing.only = others.only;
    fn.skip.only = others.only;
    fn.skip.only.failing = others.failing;
    fn.skip.each = others.each;
    fn.skip.each.failing = eachFailing;
    fn.each.failing = eachFailing;
    return fn;
}
//# sourceMappingURL=fixture.js.map