"use strict";
var fs_1 = require("fs");
var path_1 = require("path");
var curryMatch_1 = require("./curryMatch");
function fixture(ava, casesPath, baselinesPath, resultsPath) {
    if (baselinesPath && !resultsPath) {
        throw new Error('baselines and results must be specified together');
    }
    function curry(testfn) {
        return (function (title, caseName, run) {
            var hasTitle = !!run;
            if (!run) {
                // title is optional
                run = caseName;
                caseName = title;
            }
            var d = {
                casePath: path_1.resolve(casesPath, caseName)
            };
            if (baselinesPath) {
                d.baselinePath = path_1.resolve(baselinesPath, caseName);
                d.resultPath = path_1.resolve(resultsPath, caseName);
            }
            return testfn("" + (hasTitle ? title + ' ' : '') + caseName, function (t) {
                if (baselinesPath) {
                    d.match = curryMatch_1.curryMatch(d.baselinePath, d.resultPath, t);
                }
                // Inside each test, user may change the `cwd` of each case
                // so the target code can run correctly.
                // At the end of each test, restore the `cwd` to the original value.
                // doing `try-finally` and `thenable` to handle both sync and async.
                var cwd = process.cwd();
                try {
                    var result = run(t, d);
                    if (result && result.then) {
                        return result.then(function (r) {
                            process.chdir(cwd);
                            return r;
                        }, function (e) {
                            process.chdir(cwd);
                            throw e;
                        });
                    }
                }
                finally {
                    process.chdir(cwd);
                }
            });
        });
    }
    function eachCurry(testfn) {
        return function (filter, run) {
            var absCasesPath = path_1.resolve(casesPath);
            var dirs = fs_1.readdirSync(absCasesPath).toString().split(',');
            if (!run) {
                // filter is optional
                run = filter;
            }
            else {
                var regex_1 = filter instanceof RegExp ? filter : new RegExp(filter);
                dirs = dirs.filter(function (d) { return regex_1.test(d); });
            }
            var testResultPromises = dirs.map(function (caseName) {
                var d = {
                    caseName: caseName,
                    casePath: path_1.join(absCasesPath, caseName)
                };
                if (baselinesPath) {
                    d.baselinePath = path_1.resolve(baselinesPath, caseName);
                    d.resultPath = path_1.resolve(resultsPath, caseName);
                }
                return testfn(caseName, function (t) {
                    if (baselinesPath) {
                        d.match = curryMatch_1.curryMatch(d.baselinePath, d.resultPath, t);
                    }
                    // Inside each test, user may change the `cwd` of each case
                    // so the target code can run correctly.
                    // At the end of each test, restore the `cwd` to the original value.
                    // doing `try-finally` and `thenable` to handle both sync and async.
                    var cwd = process.cwd();
                    try {
                        var result = run(t, d);
                        if (result && result.then) {
                            return result.then(function (r) {
                                process.chdir(cwd);
                                return r;
                            }, function (e) {
                                process.chdir(cwd);
                                throw e;
                            });
                        }
                    }
                    finally {
                        process.chdir(cwd);
                    }
                });
            });
            // fail if any testResultPromises fail?
            return Promise.all(testResultPromises);
        };
    }
    var fn = curry(ava);
    var others = {
        failing: curry(ava.failing),
        only: curry(ava.only),
        skip: curry(ava.skip),
        todo: ava.todo,
        each: eachCurry(ava)
    };
    for (var key in others) {
        fn[key] = others[key];
    }
    var eachFailing = eachCurry(ava.failing);
    fn.failing.only = others.only;
    fn.failing.only.skip = others.skip;
    fn.failing.skip = others.skip;
    fn.failing.skip.only = others.only;
    fn.only.failing = others.failing;
    fn.only.failing.skip = others.skip;
    fn.only.skip = others.skip;
    fn.only.skip.failing = others.failing;
    fn.only.each = others.each;
    fn.only.each.failing = eachFailing;
    fn.skip.failing = others.failing;
    fn.skip.failing.only = others.only;
    fn.skip.only = others.only;
    fn.skip.only.failing = others.failing;
    fn.skip.each = others.each;
    fn.skip.each.failing = eachFailing;
    fn.each.failing = eachFailing;
    return fn;
}
exports.fixture = fixture;
//# sourceMappingURL=fixture.js.map